# Generated by Django 5.0.4 on 2024-04-22 07:28

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("api", "0008_container_port_alter_container_container_id_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="container",
            name="port",
            field=models.PositiveIntegerField(
                default=8787,
                help_text="The port on which the container's RStudio instance is accessible.",
            ),
        ),
    ]



# Generated by Django 5.0.4 on 2024-04-20 16:40

from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="Project",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "name",
                    models.CharField(
                        help_text="The name of the project.",
                        max_length=255,
                        unique=True,
                    ),
                ),
                (
                    "data_path",
                    models.CharField(
                        help_text="Filesystem path where project data is stored.",
                        max_length=1024,
                    ),
                ),
                (
                    "workspace_path",
                    models.CharField(
                        help_text="Filesystem path where R scripts are stored.",
                        max_length=1024,
                    ),
                ),
                (
                    "password",
                    models.CharField(
                        help_text="Password for the RStudio Rocker container.",
                        max_length=255,
                    ),
                ),
                (
                    "ram",
                    models.PositiveIntegerField(
                        help_text="Amount of RAM in MB allocated for the project."
                    ),
                ),
                (
                    "cpus",
                    models.PositiveIntegerField(
                        help_text="Number of CPUs allocated to the project."
                    ),
                ),
            ],
        ),
    ]



# Generated by Django 5.0.4 on 2024-04-22 07:27

import r_shepard.api.models
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("api", "0007_alter_container_container_id"),
    ]

    operations = [
        migrations.AddField(
            model_name="container",
            name="port",
            field=models.PositiveIntegerField(
                default=8787,
                help_text="The port on which the container's RStudio instance is accessible.",
                max_length=5,
            ),
        ),
        migrations.AlterField(
            model_name="container",
            name="container_id",
            field=models.CharField(
                default=r_shepard.api.models.Container.generate_container_id,
                help_text="The unique identifier of the container within the project.",
                max_length=8,
            ),
        ),
        migrations.AlterField(
            model_name="project",
            name="data_path",
            field=models.CharField(
                help_text="Filesystem path where project data is stored. Containers have read access here.",
                max_length=1024,
            ),
        ),
        migrations.AlterField(
            model_name="project",
            name="workspace_path",
            field=models.CharField(
                help_text="Filesystem path where R scripts are stored. The containers have write access here.",
                max_length=1024,
            ),
        ),
        migrations.AlterUniqueTogether(
            name="container",
            unique_together={("project", "container_id")},
        ),
    ]



# Generated by Django 5.0.4 on 2024-04-22 05:25

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("api", "0006_remove_project_cpus_remove_project_is_active_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="container",
            name="container_id",
            field=models.CharField(
                help_text="The unique identifier of the container.",
                max_length=5,
                unique=True,
            ),
        ),
    ]



# Generated by Django 5.0.4 on 2024-04-21 18:32

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("api", "0005_remove_project_url_project_local_url_and_more"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="project",
            name="cpus",
        ),
        migrations.RemoveField(
            model_name="project",
            name="is_active",
        ),
        migrations.RemoveField(
            model_name="project",
            name="local_url",
        ),
        migrations.RemoveField(
            model_name="project",
            name="ram",
        ),
        migrations.RemoveField(
            model_name="project",
            name="rstudio_version",
        ),
        migrations.RemoveField(
            model_name="project",
            name="tailscale_funnel_url",
        ),
        migrations.RemoveField(
            model_name="project",
            name="tailscale_serve_url",
        ),
        migrations.AddField(
            model_name="project",
            name="max_cpus",
            field=models.PositiveIntegerField(
                default=8, help_text="Maximum number of CPUs allocated to the project."
            ),
        ),
        migrations.AddField(
            model_name="project",
            name="max_ram",
            field=models.PositiveIntegerField(
                default=32,
                help_text="Maximum amount of RAM in GB allocated for the project.",
            ),
        ),
        migrations.CreateModel(
            name="Container",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "image",
                    models.CharField(
                        default="rocker/rstudio",
                        help_text="The name of the container image.",
                        max_length=255,
                    ),
                ),
                (
                    "tag",
                    models.CharField(
                        default="latest",
                        help_text="The tag of the container image.",
                        max_length=255,
                    ),
                ),
                (
                    "container_id",
                    models.CharField(
                        default="4Dj19",
                        help_text="The unique identifier of the container.",
                        max_length=5,
                        unique=True,
                    ),
                ),
                (
                    "ram",
                    models.PositiveIntegerField(
                        default=2,
                        help_text="Amount of RAM in GB allocated for the container.",
                    ),
                ),
                (
                    "cpus",
                    models.PositiveIntegerField(
                        default=2,
                        help_text="Number of CPUs allocated to the container.",
                    ),
                ),
                (
                    "local_url",
                    models.URLField(
                        default="",
                        help_text="Local URL to access this container's RStudio instance",
                        max_length=1024,
                    ),
                ),
                (
                    "tailscale_funnel_url",
                    models.URLField(
                        default="",
                        help_text="Tailscale Funnel URL to access this container's RStudio instance from the web",
                        max_length=1024,
                    ),
                ),
                (
                    "tailscale_serve_url",
                    models.URLField(
                        default="",
                        help_text="Tailscale Serve URL to access this container's RStudio instance from the tailnet",
                        max_length=1024,
                    ),
                ),
                (
                    "is_running",
                    models.BooleanField(
                        default=False,
                        help_text="Indicates whether the container is running",
                    ),
                ),
                (
                    "project",
                    models.ForeignKey(
                        help_text="The project to which this container belongs.",
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="containers",
                        to="api.project",
                    ),
                ),
            ],
        ),
    ]



import os
from celery import Celery

os.environ.setdefault("DJANGO_SETTINGS_MODULE", "r_shepard.settings")

app = Celery("r_shepard")
app.config_from_object("django.conf:settings", namespace="CELERY")
app.autodiscover_tasks()



"""
URL configuration for r_shepard project.

The `urlpatterns` list routes URLs to views. For more information please see:
    https://docs.djangoproject.com/en/5.0/topics/http/urls/
Examples:
Function views
    1. Add an import:  from my_app import views
    2. Add a URL to urlpatterns:  path('', views.home, name='home')
Class-based views
    1. Add an import:  from other_app.views import Home
    2. Add a URL to urlpatterns:  path('', Home.as_view(), name='home')
Including another URLconf
    1. Import the include() function: from django.urls import include, path
    2. Add a URL to urlpatterns:  path('blog/', include('blog.urls'))
"""

from django.contrib import admin
from django.urls import include, path
from django.views.generic.base import RedirectView
from two_factor.admin import AdminSiteOTPRequired
from two_factor.urls import urlpatterns as tf_urls

from r_shepard.api.views import (
    CreateContainerView,
    DeleteContainerView,
    ProjectDetailView,
    ProjectListView,
    ServeContainerView,
    StartContainerView,
    StopContainerView,
    StopServeContainerView,
)

admin.site.__class__ = AdminSiteOTPRequired

urlpatterns = [
    # Auth
    path("", include(tf_urls)),
    # Redirect empty path to /projects
    path("", RedirectView.as_view(url="/projects", permanent=True)),
    # Admin site
    path("admin/", admin.site.urls),
    # Project views
    path("projects/", ProjectListView.as_view(), name="project_list"),
    path("projects/<int:pk>/", ProjectDetailView.as_view(), name="project_detail"),
    # Container views
    path(
        "projects/<int:project_pk>/containers/new/",
        CreateContainerView.as_view(),
        name="create_container",
    ),
    path(
        "projects/<int:project_pk>/containers/<int:container_pk>/start/",
        StartContainerView.as_view(),
        name="start_container",
    ),
    path(
        "projects/<int:project_pk>/containers/<int:container_pk>/stop/",
        StopContainerView.as_view(),
        name="stop_container",
    ),
    path(
        "projects/<int:project_pk>/containers/<int:container_pk>/delete/",
        DeleteContainerView.as_view(),
        name="delete_container",
    ),
    path(
        "projects/<int:project_pk>/containers/<int:container_pk>/serve/",
        ServeContainerView.as_view(),
        name="serve_container",
    ),
    path(
        "projects/<int:project_pk>/containers/<int:container_pk>/serve/stop",
        StopServeContainerView.as_view(),
        name="stop_serve_container",
    ),
]



# Generated by Django 5.0.4 on 2024-05-08 13:12

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("api", "0017_alter_container_container_id_alter_container_cpus_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="project",
            name="description",
            field=models.CharField(
                default="",
                help_text="A short description of your project (max. 1025 characters)",
                max_length=1024,
            ),
        ),
    ]



import subprocess
from datetime import timedelta

from celery import shared_task
from django.utils import timezone

from r_shepard.api.models import Project


@shared_task
def auto_commit():
    # Only do this for projects where auto_commit is enabled
    projects = Project.objects.filter(auto_commit_enabled=True)

    for project in projects:
        # Check if it's time to commit
        try:
            now = timezone.now()
            if now - project.last_commit_time >= timedelta(
                minutes=project.commit_interval
            ):
                # Commit and push changes
                subprocess.run(["git", "add", "."], cwd=project.workspace_path)
                subprocess.run(
                    ["git", "commit", "-m", "Auto-commit"], cwd=project.workspace_path
                )
                subprocess.run(
                    ["git", "push", project.git_repo_url], cwd=project.workspace_path
                )
                # Update the last commit time
                project.last_commit_time = now
                project.save()
            else:
                print(f"Not time to commit for project {project.slug}")
        except subprocess.CalledProcessError as e:
            print(f"Encountered git error for project {project.slug}: {e}")
        except Exception as e:
            print(f"Failed to auto-commit for project {project.slug}: {e}")



from typing import Any

from django import forms
from django.contrib import admin, messages
from django.http import HttpRequest, HttpResponseRedirect
from django.http.response import HttpResponse
from django.urls import reverse

from .models import Container, Project
from .podman import PodmanError, start_container

admin.site.site_header = "R-Shepard"
admin.site.site_title = "R-Shepard"
admin.site.index_title = "Admin Area"


class EditContainerForm(forms.ModelForm):
    class Meta:
        model = Container
        fields = "__all__"
        # These fields cannot be edited and are excluded from the edit form
        exclude = [
            "project",
            "container_id",
            "image",
            "tag",
            "password",
            "port",
            "local_url",
            "tailscale_funnel_url",
            "tailscale_serve_url",
            "is_running",
        ]


class AddContainerForm(forms.ModelForm):
    password = forms.CharField(widget=forms.PasswordInput)

    class Meta:
        model = Container
        fields = "__all__"
        exclude = [
            "container_id",
            "image",
            "tag",
            "is_running",
            "port",
            "local_url",
            "tailscale_funnel_url",
            "tailscale_serve_url",
        ]


class ContainerAdmin(admin.ModelAdmin):
    def get_readonly_fields(self, request: Any, obj: Any):
        if obj:  # Edit
            return ["project", "container_id"]
        else:  # Add
            return ["container_id", "image", "tag"]

    def get_form(self, request, obj=None, **kwargs):
        if obj:
            return EditContainerForm
        else:
            return AddContainerForm

    # This is only really needed for creating containers from Admin so the logic
    # should correspond to the CreateContainerView
    def save_model(self, request: Any, obj: Any, form: Any, change: Any) -> None:
        # Calculate total resources used by all containers of the project
        total_ram = sum(container.ram for container in obj.project.containers.all())
        total_cpus = sum(container.cpus for container in obj.project.containers.all())

        # Check if the project has enough resources
        if len(obj.project.containers.all()) >= obj.project.max_containers:
            messages.error(request, "Maximum number of containers reached.")
            return
        if (
            total_ram + obj.ram > obj.project.max_ram
            or total_cpus + obj.cpus > obj.project.max_cpus
            or len(obj.project.containers.all()) >= obj.project.max_containers
        ):
            messages.error(request, "Not enough resources.")
            return

        # Start podman container
        try:
            start_container(obj.project, obj, obj.password)
            obj.is_running = True
        except PodmanError as e:
            messages.error(request, str(e))
            return
        super().save_model(request, obj, form, change)

    # Redirect to project detail page after adding a container
    def response_add(self, request, obj, post_url_continue=None):
        return HttpResponseRedirect(reverse("project_detail", args=[obj.project.pk]))

    def response_change(self, request: HttpRequest, obj: Any) -> HttpResponse:
        return HttpResponseRedirect(reverse("project_detail", args=[obj.project.pk]))


class ProjectAddForm(forms.ModelForm):
    class Meta:
        model = Project
        fields = [
            "name",
            "description",
            "max_containers",
            "max_ram",
            "max_cpus",
            "auto_commit_enabled",
            "git_repo_url",
            "commit_interval",
        ]


class ProjectChangeForm(forms.ModelForm):
    class Meta:
        model = Project
        fields = "__all__"


class ProjectAdmin(admin.ModelAdmin):
    def get_form(self, request, obj=None, **kwargs):
        if obj:
            return ProjectChangeForm
        else:
            return ProjectAddForm

    def response_add(self, request, obj, post_url_continue=None):
        return HttpResponseRedirect(reverse("project_list"))

    def response_change(self, request: HttpRequest, obj: Any) -> HttpResponse:
        return HttpResponseRedirect(reverse("project_detail", args=[obj.pk]))


admin.site.register(Container, ContainerAdmin)
admin.site.register(Project, ProjectAdmin)



# Generated by Django 5.0.4 on 2024-05-10 09:10

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("api", "0018_project_description"),
    ]

    operations = [
        migrations.AlterField(
            model_name="project",
            name="commit_interval",
            field=models.PositiveIntegerField(
                default=15, help_text="Commit interval in minutes.", null=True
            ),
        ),
        migrations.AlterField(
            model_name="project",
            name="git_repo_url",
            field=models.CharField(
                blank=True,
                help_text="URL of the Git repository.",
                max_length=255,
                null=True,
                validators=[
                    django.core.validators.RegexValidator(
                        "(git@|http:\\/\\/|https:\\/\\/)([\\w\\.@:]+)(\\/|:)([\\w\\.\\/\\-]+\\.git)",
                        "Enter a valid Git URL.",
                    )
                ],
                verbose_name="Git Repository URL",
            ),
        ),
        migrations.AlterField(
            model_name="project",
            name="max_containers",
            field=models.PositiveIntegerField(
                default=4,
                help_text="Maximum number of containers that can be run for the project.",
                verbose_name="Max. number of containers",
            ),
        ),
        migrations.AlterField(
            model_name="project",
            name="max_cpus",
            field=models.PositiveIntegerField(
                default=8,
                help_text="Maximum number of CPUs allocated to the project.",
                verbose_name="Max. CPUs",
            ),
        ),
        migrations.AlterField(
            model_name="project",
            name="max_ram",
            field=models.PositiveIntegerField(
                default=32,
                help_text="Maximum amount of RAM in GB allocated for the project.",
                verbose_name="Max. RAM",
            ),
        ),
        migrations.AlterField(
            model_name="project",
            name="name",
            field=models.CharField(
                help_text="The name of the project.",
                max_length=255,
                unique=True,
                validators=[
                    django.core.validators.RegexValidator(
                        "^[\\w\\s]+$",
                        "Enter a valid project name. This value may contain only letters, numbers, and whitespace.",
                    )
                ],
            ),
        ),
    ]
